@@ -2,6 +2,7 @@
 * GNU LibreJS - A browser add-on to block nonfree nontrivial JavaScript.
 *
 * Copyright (C) 2018 Giorgio Maone <giorgio@maone.net>
+* Copyright (C) 2022 Yuchen Pei <id@ypei.org>
 *
 * This file is part of GNU LibreJS.
 *
@@ -25,45 +26,45 @@
 
 'use strict';
 
-let licensesByLabel = new Map();
-let licensesByUrl = new Map();
-{
-  let { licenses } = require('../license_definitions');
-  let mapByLabel = (label, license) => licensesByLabel.set(label.toUpperCase(), license);
-  for (let [id, l] of Object.entries(licenses)) {
-    let { identifier, canonicalUrl, licenseName } = l;
-    if (identifier) {
-      mapByLabel(identifier, l);
-    } else {
-      l.identifier = id;
-    }
-    if (id !== identifier) {
-      mapByLabel(id, l);
-    }
-    if (licenseName) {
-      mapByLabel(licenseName, l);
-    }
-    if (Array.isArray(canonicalUrl)) {
-      for (let url of canonicalUrl) {
-        licensesByUrl.set(url, l);
-      }
-    }
+const licenses = require('../common/license_definitions.json')
+const licensesByLabel = new Map(Object.entries(licenses).map(([id, license]) =>
+  [
+    [license.identifier.toUpperCase(), license],
+    [id.toUpperCase(), license],
+    [license.licenseName.toUpperCase(), license]
+  ]).flat());
+const licensesByUrl = new Map(Object.values(licenses).map(license =>
+  license.canonicalUrl.map(url => [url, license])).flat());
+for (const [id, license] of Object.entries(licenses)) {
+  if (!license.identifier) {
+    license.identifier = id;
   }
 }
 
-let cachedHrefs = new Map();
+const cachedHrefs = new Map();
 
-var ExternalLicenses = {
+const ExternalLicenses = {
   purgeCache(tabId) {
     cachedHrefs.delete(tabId);
   },
 
+  /**
+   * Checks external script using web labels
+   * 
+   * If the script cannot be found in the web labels table, returns null.
+   * 
+   * If the script can be found in the web labels table, and at least
+   * one of its licenses can be found in our free license DB, returns
+   * "free".
+   *
+   * Otherwise returns "nonfree".
+   */
   async check(script) {
-    let { url, tabId, frameId, documentUrl } = script;
-    let tabCache = cachedHrefs.get(tabId);
-    let frameCache = tabCache && tabCache.get(frameId);
-    let cache = frameCache && frameCache.get(documentUrl);
-    let scriptInfo = await browser.tabs.sendMessage(tabId, {
+    const { url, tabId, frameId, documentUrl } = script;
+    const tabCache = cachedHrefs.get(tabId);
+    const frameCache = tabCache && tabCache.get(frameId);
+    const cache = frameCache && frameCache.get(documentUrl);
+    const scriptInfo = await browser.tabs.sendMessage(tabId, {
       action: 'checkLicensedScript',
       url,
       cache,
@@ -72,26 +73,13 @@ var ExternalLicenses = {
     if (!(scriptInfo && scriptInfo.licenseLinks.length)) {
       return null;
     }
-    scriptInfo.licenses = new Set();
-    scriptInfo.toString = function() {
-      let licenseIds = [...this.licenses].map(l => l.identifier).sort().join(', ');
-      return licenseIds
-        ? `Free license${this.licenses.size > 1 ? 's' : ''} (${licenseIds})`
-        : 'Unknown license(s)';
-    }
-    let match = (map, key) => {
-      if (map.has(key)) {
-        scriptInfo.licenses.add(map.get(key));
-        return true;
-      }
-      return false;
-    };
-
-    for (let { label, url } of scriptInfo.licenseLinks) {
-      match(licensesByLabel, label = label.trim().toUpperCase()) ||
-        match(licensesByUrl, url) ||
-        match(licensesByLabel, label.replace(/^GNU-|-(?:OR-LATER|ONLY)$/, ''));
-    }
+    scriptInfo.licenses = new Set(scriptInfo.licenseLinks.map(
+      ({ label, url }) => {
+        const uLabel = label.trim().toUpperCase();
+        const license = licensesByLabel.get(uLabel) || licensesByUrl.get(url) ||
+          licensesByLabel.get(uLabel.replace(/^GNU-|-(?:OR-LATER|ONLY)$/, ''));
+        return license ? [license] : [];
+      }).flat());
     scriptInfo.free = scriptInfo.licenses.size > 0;
     return scriptInfo;
   },
@@ -104,34 +92,33 @@ var ExternalLicenses = {
   * Returns true if the document has been actually modified, false otherwise.
   */
   optimizeDocument(doc, cachePointer) {
-    let cache = {};
-    let { tabId, frameId, documentUrl } = cachePointer;
-    let frameCache = cachedHrefs.get(tabId);
-    if (!frameCache) {
-      cachedHrefs.set(tabId, frameCache = new Map());
-    }
+    const cache = {};
+    const { tabId, frameId, documentUrl } = cachePointer;
+    const frameCache = cachedHrefs.get(tabId) || new Map();
+    cachedHrefs.set(tabId, frameCache);
     frameCache.set(frameId, new Map([[documentUrl, cache]]));
 
-    let link = doc.querySelector('link[rel="jslicense"], link[data-jslicense="1"], a[rel="jslicense"], a[data-jslicense="1"]');
+    const link = doc.querySelector('link[rel="jslicense"], link[data-jslicense="1"], a[rel="jslicense"], a[data-jslicense="1"]');
     if (link) {
-      let href = link.getAttribute('href');
+      const href = link.getAttribute('href');
       cache.webLabels = { href };
-      let move = () => !!doc.head.insertBefore(link, doc.head.firstChild);
+      const move = (link) => !!doc.head.insertBefore(link, doc.head.firstChild);
       if (link.parentNode === doc.head) {
+        // TODO: eliminate let
         let node = link.previousElementSibling;
         for (; node; node = node.previousElementSibling) {
           if (node.tagName.toUpperCase() === 'SCRIPT') {
-            return move();
+            return move(link);
           }
         }
       } else { // the reference is only in the body
         if (link.tagName.toUpperCase() === 'A') {
-          let newLink = doc.createElement('link');
+          const newLink = doc.createElement('link');
           newLink.rel = 'jslicense';
           newLink.setAttribute('href', href);
-          link = newLink;
+          return move(newLink);
         }
-        return move();
+        return move(link);
       }
     }
 
