@@ -2,6 +2,7 @@
 * GNU LibreJS - A browser add-on to block nonfree nontrivial JavaScript.
 *
 * Copyright (C) 2018 Giorgio Maone <giorgio@maone.net>
+* Copyright (C) 2022 Yuchen Pei <id@ypei.org>
 *
 * This file is part of GNU LibreJS.
 *
@@ -21,36 +22,49 @@
 'use strict';
 
 describe('LibreJS\' components', () => {
-  let LibreJS = browser.extension.getBackgroundPage().LibreJS;
-  let license = {
+  const LibreJS = browser.extension.getBackgroundPage().LibreJS;
+  const license = {
     id: 'GPL-3.0',
     url: 'http://www.gnu.org/licenses/gpl-3.0.html',
     magnet: 'magnet:?xt=urn:btih:1f739d935676111cfff4b4693e3816e664797050&dn=gpl-3.0.txt',
   };
-  let unknownLicense = {
+  const unknownLicense = {
     id: 'Acme-proprietary-1.5',
     url: 'http://www.acme.com/license-1.5.html',
     magnet: 'magnet:?xt=urn:btih:2f739d935676111cfff4b4693e3816e664797050&dn=acme-1.5.txt'
   };
 
-  let trivial = '1+1';
-  let nontrivial = 'function nt() { document.documentElement.innerHTML=""; nt(); }';
-  let licensed = `// @license ${license.magnet} ${license.id}\n${nontrivial}\n// @license-end`;
-  let unknownLicensed = `// @license ${unknownLicense.magnet} ${unknownLicense.id}\n${nontrivial}\n// @license-end`;
-  let malformedLicensed = `// @license\n${nontrivial}`;
+  const trivial = '1+1';
+  const nontrivial = 'function nt() { document.documentElement.innerHTML=""; nt(); }';
+  // code calling reserved object is nontrivial
+  const nontrivialCall = 'eval();';
+  // code calling anything else is trivial
+  const trivialCall = 'foo();';
+  const licensed = `// @license ${license.magnet} ${license.id}\n${nontrivial}\n// @license-end`;
+  const licensedStarCommented = `/* @license ${license.magnet} ${license.id} */${nontrivial}/* @license-end */`;
+  const unknownLicensed = `// @license ${unknownLicense.magnet} ${unknownLicense.id}\n${nontrivial}\n// @license-end`;
+  const commentedOutUnknownLicensed =
+    unknownLicensed.split('\n').map(y => '// ' + y).join('\n');
+  const malformedLicensed = `// @license\n${nontrivial}`;
+  const commentedOutMalformedLicensed =
+    malformedLicensed.split('\n').map(y => '// ' + y).join('\n');
+  const emptyLicensed = `// @license ${license.magnet} ${license.id}\n// @license-end`;
+  const licensedNontrivialTrivial = `${emptyLicensed}\n${nontrivial}\n${trivial}`;
+
   let tab, documentUrl;
+  const enableContactFinderTests = false;
 
   beforeAll(async () => {
-    let url = browser.extension.getURL('/test/resources/index.html');
+    const url = browser.extension.getURL('/test/resources/index.html');
     tab = (await browser.tabs.query({ url }))[0] || (await browser.tabs.create({ url }));
     documentUrl = url;
 
   });
 
   describe('The whitelist/blacklist manager', () => {
-    let { ListManager, ListStore, Storage } = LibreJS;
-    let lm = new ListManager(new ListStore('_test.whitelist', Storage.CSV), new ListStore('_test.blacklist', Storage.CSV), new Set());
-    let forgot = ['http://formerly.whitelist.ed/', 'http://formerly.blacklist.ed/'];
+    const { ListManager, ListStore, Storage } = LibreJS;
+    const lm = new ListManager(new ListStore('_test.whitelist', Storage.CSV), new ListStore('_test.blacklist', Storage.CSV), new Set());
+    const forgot = ['http://formerly.whitelist.ed/', 'http://formerly.blacklist.ed/'];
 
     beforeAll(async () => {
       await lm.whitelist('https://fsf.org/*', 'https://*.gnu.org/*', forgot[0]);
@@ -63,7 +77,7 @@ describe('LibreJS\' components', () => {
 
       await lm.forget(...forgot);
 
-      for (let url of forgot) {
+      for (const url of forgot) {
         expect(lm.getStatus(url)).toBe('unknown');
       }
     });
@@ -86,10 +100,10 @@ describe('LibreJS\' components', () => {
   })
 
   describe('The external script source processor', () => {
-    let url = 'https://www.gnu.org/mock-script.js';
+    const url = 'https://www.gnu.org/mock-script.js';
 
-    let processScript = async (source, whitelisted = false) =>
-      await LibreJS.handle_script({
+    const processScript = async (source, whitelisted = false) =>
+      await LibreJS.handleScript({
         text: source,
         request: { url, tabId: tab.id, documentUrl, frameId: 0 },
       }, whitelisted);
@@ -99,45 +113,57 @@ describe('LibreJS\' components', () => {
     });
 
     it('should block trivial scripts too', async () => {
-      let processed = await processScript(trivial);
+      const processed = await processScript(trivial);
       expect(processed || trivial).not.toContain(trivial);
     });
 
     it('should block non-trivial scripts', async () => {
-      let processed = await processScript(nontrivial);
+      const processed = await processScript(nontrivial);
       expect(processed || nontrivial).not.toContain(nontrivial);
     });
 
     it('should accept scripts with known free license tags', async () => {
-      let processed = await processScript(licensed);
+      const processed = await processScript(licensed);
       expect(processed || licensed).toContain(nontrivial);
+      const processed1 = await processScript(licensedStarCommented);
+      expect(processed1 || licensed).toContain(nontrivial);
     });
 
     it('should block scripts with unknown license tags', async () => {
-      let processed = await processScript(unknownLicensed);
+      const processed = await processScript(unknownLicensed);
       expect(processed).not.toContain(nontrivial);
     });
 
+    it('should leave alone scripts with commented out unknown license tags', async () => {
+      const processed = await processScript(commentedOutUnknownLicensed);
+      expect(processed).toContain(nontrivial);
+    });
+
     it('should block scripts with malformed license tags', async () => {
-      let processed = await processScript(malformedLicensed);
+      const processed = await processScript(malformedLicensed);
       expect(processed).not.toContain(nontrivial);
     });
+
+    it('should leave alone scripts with commented out malformed license tags', async () => {
+      const processed = await processScript(commentedOutMalformedLicensed);
+      expect(processed).toContain(nontrivial);
+    });
   });
 
   describe('The HTML processor', () => {
-    let processHtml =
+    const processHtml =
       async (html, whitelisted = false) =>
         LibreJS.editHtml(html, tab.url, tab.id, 0, whitelisted);
 
-    let addScript = (html, script, before = '</head>') =>
+    const addScript = (html, script, before = '</head>') =>
       html.replace(before, `<script>${script}</script>${before}`);
 
-    let addToBody = (html, fragment) => html.replace('</body>', `${fragment}</body>`);
+    const addToBody = (html, fragment) => html.replace('</body>', `${fragment}</body>`);
 
-    let jsUrl = js => `javascript:${encodeURIComponent(js)}`;
+    const jsUrl = js => `javascript:${encodeURIComponent(js)}`;
 
     function extractScripts(html, def = '') {
-      let matches = html && html.match(/<script>[^]*?<\/script>/g);
+      const matches = html && html.match(/<script>[^]*?<\/script>/g);
       return matches && matches.join('') || def;
     }
 
@@ -159,89 +185,98 @@ describe('LibreJS\' components', () => {
     });
 
     it('should accept trivial scripts', async () => {
-      let trivialInHtml = addScript(html, trivial);
-      let processed = await processHtml(trivialInHtml);
-      expect(extractScripts(processed, trivial)).toContain(trivial);
+      expect(extractScripts(await processHtml(
+        addScript(html, trivial)), trivial)).toContain(trivial);
+      expect(extractScripts(await processHtml(
+        addScript(html, trivialCall)), trivialCall)).toContain(trivialCall);
     });
 
     it('should block non-trivial scripts', async () => {
-      let processed = await processHtml(nontrivialInHtml);
-      expect(extractScripts(processed, nontrivial)).not.toContain(nontrivial);
+      expect(extractScripts(await processHtml(
+        nontrivialInHtml), nontrivial)).not.toContain(nontrivial);
+      expect(extractScripts(await processHtml(
+        addScript(html, nontrivialCall)), nontrivialCall)).not.toContain(nontrivialCall);
     });
 
     it('should accept scripts with known free license tags', async () => {
-      let licensedInHtml = addScript(html, licensed);
-      let processed = await processHtml(licensedInHtml);
+      const licensedInHtml = addScript(html, licensed);
+      const processed = await processHtml(licensedInHtml);
       expect(extractScripts(processed, licensed)).toContain(nontrivial);
     });
 
     it('should block scripts with unknown license tags', async () => {
-      let unknownInHtml = addScript(html, unknownLicensed);
-      let processed = await processHtml(unknownInHtml);
+      const unknownInHtml = addScript(html, unknownLicensed);
+      const processed = await processHtml(unknownInHtml);
       expect(extractScripts(processed, nontrivial)).not.toContain(nontrivial);
     });
 
     it('should block scripts with malformed license tags', async () => {
-      let malformedInHtml = addScript(html, malformedLicensed);
-      let processed = await processHtml(malformedInHtml);
+      const malformedInHtml = addScript(html, malformedLicensed);
+      const processed = await processHtml(malformedInHtml);
+      expect(extractScripts(processed, nontrivial)).not.toContain(nontrivial);
+    });
+
+    it('should block the unlicensed nontrivial part sandwiched between licensed and trivial parts', async () => {
+      const modifiedHtml = addScript(html, licensedNontrivialTrivial);
+      const processed = await processHtml(modifiedHtml);
       expect(extractScripts(processed, nontrivial)).not.toContain(nontrivial);
     });
 
     it('should accept scripts on globally licensed pages', async () => {
-      let globalLicense = `/* @licstart The following is the entire license notice
+      const globalLicense = `/* @licstart The following is the entire license notice
         for the JavaScript code in this page.
         -- Some free license --
         @licend The above is the entire license notice for the JavaScript code in this page. */`;
 
-      let licensed = addScript(nontrivialInHtml, globalLicense, '<script>');
-      let processed = await processHtml(html);
+      const licensed = addScript(nontrivialInHtml, globalLicense, '<script>');
+      const processed = await processHtml(html);
       expect(extractScripts(processed, licensed)).toContain(nontrivial);
     });
 
     it('should discriminate trivial, non-trivial and licensed mixed on the same page', async () => {
       let mixedPage = addScript(addScript(nontrivialInHtml, trivial), licensed);
-      let processed = await processHtml(mixedPage);
+      const processed = await processHtml(mixedPage);
       expect(processed).not.toBeNull();
-      let scripts = extractScripts(processed, nontrivial);
+      const scripts = extractScripts(processed, nontrivial);
       expect(scripts).toContain(trivial);
       expect(scripts).toContain(licensed);
       expect(scripts.replace(licensed, '')).not.toContain(nontrivial);
     });
 
     it('should correctly process (de)duplicated inline scripts', async () => {
-      let trivialAsUrl = jsUrl(trivial);
-      let nontrivialAsUrl = jsUrl(nontrivial);
-      let a = (url, label) => `<a href="${url}">${label}</a>`;
+      const trivialAsUrl = jsUrl(trivial);
+      const nontrivialAsUrl = jsUrl(nontrivial);
+      const a = (url, label) => `<a href="${url}">${label}</a>`;
       let mixedPage = '<body></body>';
       for (let dup = 0; dup < 3; dup++) {
         mixedPage = addToBody(mixedPage, a(trivialAsUrl, `Trivial #${dup}`));
         mixedPage = addToBody(mixedPage, a(nontrivialAsUrl, `Nontrivial #${dup}`));
       }
-      let processed = await processHtml(mixedPage);
+      const processed = await processHtml(mixedPage);
       expect(processed).not.toBeNull();
       expect(processed).toContain(trivialAsUrl);
       expect(processed).not.toContain(nontrivialAsUrl);
     });
 
     it('should force displaying NOSCRIPT elements (except those with @data-librejs-nodisplay) where scripts have been blocked', async () => {
-      let noscriptContent = 'I\'m NOSCRIPT content';
-      let asNoscript = `<noscript>${noscriptContent}</noscript>`;
-      let asNodisplay = `<noscript data-librejs-nodisplay>${noscriptContent}</noscript>`;
-      let asSpan = `<span>${noscriptContent}</span>`;
-      let page = addToBody(addToBody(nontrivialInHtml, asNoscript), asNodisplay);
-      let processed = await processHtml(page);
+      const noscriptContent = 'I\'m NOSCRIPT content';
+      const asNoscript = `<noscript>${noscriptContent}</noscript>`;
+      const asNodisplay = `<noscript data-librejs-nodisplay>${noscriptContent}</noscript>`;
+      const asSpan = `<span>${noscriptContent}</span>`;
+      const page = addToBody(addToBody(nontrivialInHtml, asNoscript), asNodisplay);
+      const processed = await processHtml(page);
       expect(processed).not.toContain(asNoscript);
       expect(processed).toContain(asSpan);
       expect(processed).not.toContain(asNodisplay);
     });
 
     it('should always force displaying @data-librejs-display elements', async () => {
-      let content = 'I\'m FORCED content';
-      let asDisplay = `<span data-librejs-display>${content}</span>`;
-      let asSpan = `<span>${content}</span>`;
+      const content = 'I\'m FORCED content';
+      const asDisplay = `<span data-librejs-display>${content}</span>`;
+      const asSpan = `<span>${content}</span>`;
       for (let page of [nontrivialInHtml, '<body></body>']) {
         page = addToBody(page, asDisplay);
-        let processed = await processHtml(page);
+        const processed = await processHtml(page);
         expect(processed).not.toContain(asDisplay);
         expect(processed).not.toContain(asSpan);
       }
@@ -249,12 +284,12 @@ describe('LibreJS\' components', () => {
   });
 
   describe('The external (Web Labels) license checker', () => {
-    let { ExternalLicenses } = LibreJS;
+    const { ExternalLicenses } = LibreJS;
     let check;
 
     beforeAll(async () => {
-      let args = { tabId: tab.id, frameId: 0, documentUrl };
-      let resolve = url => new URL(url, documentUrl).href;
+      const args = { tabId: tab.id, frameId: 0, documentUrl };
+      const resolve = url => new URL(url, documentUrl).href;
       check = async url => await ExternalLicenses.check(Object.assign({ url: resolve(url) }, args));
       await browser.tabs.executeScript(tab.id, {
         file: '/content/externalLicenseChecker.js'
@@ -262,26 +297,72 @@ describe('LibreJS\' components', () => {
     });
 
     it('should recognize free licenses', async () => {
-      let scriptInfo = await check('jquery.js');
+      const scriptInfo = await check('jquery.js');
       console.debug(scriptInfo);
       expect(scriptInfo.free).toBeTruthy();
     });
     it('should accept scripts if any of multiple licenses is free', async () => {
-      let scriptInfo = await check('app-trilicensed.js');
+      const scriptInfo = await check('app-trilicensed.js');
       console.debug(scriptInfo);
       expect(scriptInfo.free).toBeTruthy();
     });
     it('should block scripts declaring only proprietary license(s)', async () => {
-      let scriptInfo = await check('proprietary.js');
+      const scriptInfo = await check('proprietary.js');
       console.debug(scriptInfo);
       expect(scriptInfo.free).toBeFalsy();
     });
     it('should block scripts not declaring any license', async () => {
-      let scriptInfo = await check('tracker.js');
+      const scriptInfo = await check('tracker.js');
       console.debug(scriptInfo);
       expect(scriptInfo).toBeNull();
     });
   });
+
+  if (enableContactFinderTests) {
+    describe('The contact finder', () => {
+      it('should display the contact finder iframe', async (done) => {
+        await browser.runtime.connect({ name: "port-from-cs" }).postMessage({ invoke_contact_finder: 1 });
+        // Direct await / async does not work as executeScript does not wait
+        // for the listener
+        setTimeout(async () => {
+          const frame = await browser.tabs.executeScript(tab.id, {
+            code: 'document.getElementById("_LibreJS_frame").outerHTML'
+          })
+          expect(frame).toBeTruthy();
+          done();
+        }, 100);
+      });
+      it('should display the correct contact info in the contact finder iframe', async (done) => {
+        await browser.runtime.connect({ name: "port-from-cs" }).postMessage({ invoke_contact_finder: 1 });
+        // Direct await / async does not work as executeScript does not wait
+        // for the listener
+        setTimeout(async () => {
+          const [frameBody] = await browser.tabs.executeScript(tab.id, {
+            code: 'document.getElementById("_LibreJS_frame").contentWindow.document.body.innerHTML'
+          });
+          expect(frameBody).not.toContain('About Us');
+          expect(frameBody).toContain('Contact Us');
+          expect(frameBody).toContain('Website Feedback');
+          expect(frameBody).toContain('lib@re.js');
+          done();
+        }, 200);
+      });
+    });
+  }
+
+  describe('The prefs', () => {
+    it('should have the defaults', async () => {
+      const defaults = await browser.storage.local.get(['pref_subject', 'pref_body']);
+      expect(defaults).toEqual({
+        pref_subject: 'Issues with Javascript on your website',
+        pref_body: `Please consider using a free license for the Javascript on your website.
+
+[Message generated by LibreJS. See https://www.gnu.org/software/librejs/ for more information]
+`
+      });
+    });
+  });
+
   afterAll(async () => {
     await browser.tabs.remove(tab.id);
     browser.tabs.update((await browser.tabs.getCurrent()).id, { active: true });
